"""Kubetest wrapper for the Kubernetes ``Ingress`` API Object."""

import logging

from kubernetes import client

from kubetest import condition, utils

from .api_object import ApiObject

log = logging.getLogger("kubetest")


class Ingress(ApiObject):
    """Kubetest wrapper around a Kubernetes `Ingress`_ API Object.

    The actual ``kubernetes.client.ExtensionsV1beta1Api`` instance that this
    wraps can be accessed via the ``obj`` instance member.

    This wrapper provides some convenient functionality around the
    API Object and provides some state management for the `Ingress`_.

    .. _Ingress:
        https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.18/#ingress-v1beta1-extensions
    """

    obj_type = client.ExtensionsV1beta1Api

    api_clients = {
        "preferred": client.ExtensionsV1beta1Api,
        "extensions/v1beta1": client.ExtensionsV1beta1Api,
    }

    def __str__(self):
        return str(self.obj)

    def __repr__(self):
        return self.__str__()

    def create(self, namespace: str = None) -> None:
        """Create the Ingress under the given namespace.

        Args:
            namespace: The namespace to create the Ingress under.
                If the Ingress was loaded via the kubetest client, the
                namespace will already be set, so it is not needed here.
                Otherwise, the namespace will need to be provided.
        """
        if namespace is None:
            namespace = self.namespace

        log.info('creating ingress "%s" in namespace "%s"', self.name, self.namespace)
        log.debug("ingress: %s", self.obj)

        self.obj = self.api_client.create_namespaced_ingress(
            namespace=namespace,
            body=self.obj,
        )

    def delete(self, options: client.V1DeleteOptions = None) -> client.V1Status:
        """Delete the Ingress.

        This method expects the Ingress to have been loaded or otherwise
        assigned a namespace already. If it has not, the namespace will need
        to be set manually.

        Args:
             options: Options for Ingress deletion.

        Returns:
            The status of the delete operation.
        """
        if options is None:
            options = client.V1DeleteOptions()

        log.info('deleting ingress "%s"', self.name)
        log.debug("delete options: %s", options)
        log.debug("ingress: %s", self.obj)

        return self.api_client.delete_namespaced_ingress(
            name=self.name,
            namespace=self.namespace,
            body=options,
        )

    def refresh(self) -> None:
        """Refresh the underlying Kubernetes Ingress resource."""
        self.obj = self.api_client.read_namespaced_ingress(
            name=self.name,
            namespace=self.namespace,
        )

    def is_ready(self) -> bool:
        """Check if the Ingress is in the ready state.

        Returns:
            True if in the ready state; False otherwise.
        """
        try:
            self.refresh()
        except:  # noqa
            return False
        else:
            return True

    def has_load_balancer_ingress(self) -> bool:
        """Check if the ingress has been assigned an ingress.

        Returns:
            True if an ingress has been assigned; False otherwise.
        """
        self.refresh()
        return self.obj.status.load_balancer.ingress is not None

    def wait_for_load_balancer_ingress(self, timeout: int = None) -> None:
        """Wait until the ingress has been assigned an ingress.

        Args:
            timeout: The maximum time to wait in seconds, for the
            Ingress to be assigned an ingress. If unspecified,
            this will wait indefinitely. If specified and the timeout
            is met or exceeded, a TimeoutError will be raised.

        Raises:
            TimeoutError: The specified timeout was exceeded.
        """
        wait_condition = condition.Condition(
            "Ingress has been assigned an ingress", self.has_load_balancer_ingress
        )

        utils.wait_for_condition(condition=wait_condition, timeout=timeout, interval=1)



"""Kubetest wrapper for the Kubernetes ``ServiceAccount`` API Object."""

import logging

from kubernetes import client

from kubetest.objects import ApiObject

log = logging.getLogger("kubetest")


class ServiceAccount(ApiObject):
    """Kubetest wrapper around a Kubernetes `ServiceAccount`_ API Object.
    The actual ``kubernetes.client.V1ServiceAccount`` instance that this
    wraps can be accessed via the ``obj`` instance member.
    This wrapper provides some convenient functionality around the
    API Object and provides some state management for the `ServiceAccount`_.
    .. _ServiceAccount:
        https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.18/#serviceaccount-v1-core
    """

    obj_type = client.V1ServiceAccount

    api_clients = {
        "preferred": client.CoreV1Api,
        "v1": client.CoreV1Api,
    }

    def create(self, name: str = None) -> None:
        """Create the ServiceAccount under the given name.
        Args:
            name: The name to create the ServiceAccount under. If the
                name is not provided, it will be assumed to already be
                in the underlying object spec. If it is not, namespace
                operations will fail.
        """
        if name is not None:
            self.name = name

        log.info(f'creating serviceaccount "{self.name}"')
        log.debug(f"serviceaccount: {self.obj}")

        self.obj = self.api_client.create_namespaced_service_account(
            body=self.obj,
            namespace=self.namespace,
        )

    def delete(self, options: client.V1DeleteOptions = None) -> client.V1Status:
        """Delete the Namespace.
        Args:
             options: Options for ServiceAccount deletion.
        Returns:
            The status of the delete operation.
        """
        if options is None:
            options = client.V1DeleteOptions()

        log.info(f'deleting ServiceAccount "{self.name}"')
        log.debug(f"delete options: {options}")
        log.debug(f"service account: {self.obj}")

        return self.api_client.delete_namespaced_service_account(
            name=self.name, namespace=self.namespace, body=options
        )

    def refresh(self) -> None:
        """Refresh the underlying Kubernetes ServiceAccount resource."""
        self.obj = self.api_client.read_namespaced_service_account(
            name=self.name,
            namespace=self.namespace,
        )

    def is_ready(self) -> bool:
        """Check if the ServiceAccount is in the ready state.

        ServiceAccount do not have a "status" field to check, so we will
        measure their readiness status by whether or not they exist
        on the cluster.

        Returns:
            True if in the ready state; False otherwise.
        """
        try:
            self.refresh()
        except:  # noqa
            return False
        else:
            return True



"""kubetest -- a Kubernetes integration test framework in Python."""

__title__ = "kubetest"
__version__ = "0.9.4"
__description__ = "A Kubernetes integration test framework in Python."
__author__ = "Vapor IO"
__author_email__ = "vapor@vapor.io"
__url__ = "https://github.com/vapor-ware/kubetest"
__license__ = "GNU General Public License v3.0"



"""Kubetest wrapper for the Kubernetes ``NetworkPolicy`` API Object."""

import logging

from kubernetes import client

from .api_object import ApiObject

log = logging.getLogger("kubetest")


class NetworkPolicy(ApiObject):
    """Kubetest wrapper around a Kubernetes `NetworkPolicy`_ API Object.

    The actual ``kubernetes.client.NetworkingV1Api`` instance that this
    wraps can be accessed via the ``obj`` instance member.

    This wrapper provides some convenient functionality around the
    API Object and provides some state management for the `NetworkPolicy`_.

    .. _NetworkPolicy:
        https://kubernetes.io/docs/reference/generated/kubernetes-api/v1.18/#networkpolicy-v1-networking-k8s-io
    """

    obj_type = client.V1NetworkPolicy

    api_clients = {
        "preffered": client.NetworkingV1Api,
        "networking.k8s.io/v1": client.NetworkingV1Api,
    }

    def __str__(self):
        return str(self.obj)

    def __repr__(self):
        return self.__str__()

    def create(self, namespace: str = None) -> None:
        """Create the NetworkPolicy under the given namespace.

        Args:
            namespace: The namespace to create the NetworkPolicy under.
                If the NetworkPolicy was loaded via the kubetest client, the
                namespace will already be set, so it is not needed here.
                Otherwise, the namespace will need to be provided.
        """
        if namespace is None:
            namespace = self.namespace

        log.info(
            'creating networkpolicy "%s" in namespace "%s"', self.name, self.namespace
        )
        log.debug("network_policy: %s", self.obj)

        self.obj = self.api_client.create_namespaced_network_policy(
            namespace=namespace,
            body=self.obj,
        )

    def delete(self, options: client.V1DeleteOptions = None) -> client.V1Status:
        """Delete the NetworkPolicy.

        This method expects the NetworkPolicy to have been loaded or otherwise
        assigned a namespace already. If it has not, the namespace will need
        to be set manually.

        Args:
             options: Options for NetworkPolicy deletion.

        Returns:
            The status of the delete operation.
        """
        if options is None:
            options = client.V1DeleteOptions()

        log.info('deleting network_policy "%s"', self.name)
        log.debug("delete options: %s", options)
        log.debug("network_policy: %s", self.obj)

        return self.api_client.delete_namespaced_network_policy(
            name=self.name,
            namespace=self.namespace,
            body=options,
        )

    def refresh(self) -> None:
        """Refresh the underlying Kubernetes NetworkPolicy resource."""
        self.obj = self.api_client.read_namespaced_network_policy(
            name=self.name,
            namespace=self.namespace,
        )

    def is_ready(self) -> None:
        """Check if the NetworkPolicy is in the ready state.

        Returns:
            True if in the ready state; False otherwise.
        """
        try:
            self.refresh()
        except:  # noqa
            return False
        else:
            return True



"""Setup and packaging for kubetest."""

import os
from codecs import open  # for consistent encoding

from setuptools import find_packages, setup

here = os.path.abspath(os.path.dirname(__file__))

# Load the package's __init__.py file as a dictionary.
pkg = {}
with open(os.path.join(here, "kubetest", "__init__.py"), "r", "utf-8") as f:
    exec(f.read(), pkg)

# Load the README
readme = ""
if os.path.exists("README.md"):
    with open("README.md", "r", "utf-8") as f:
        readme = f.read()

setup(
    name=pkg["__title__"],
    version=pkg["__version__"],
    description=pkg["__description__"],
    long_description=readme,
    long_description_content_type="text/markdown",
    url=pkg["__url__"],
    author=pkg["__author__"],
    author_email=pkg["__author_email__"],
    license=pkg["__license__"],
    packages=find_packages(),
    python_requires=">=3.6",
    package_data={
        "": ["LICENSE"],
    },
    install_requires=[
        "kubernetes",
        "pyyaml>=4.2b1",
        "pytest",
    ],
    zip_safe=False,
    classifiers=[
        "Environment :: Plugins",
        "Framework :: Pytest",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
    ],
    # this make a plugin available to pytest
    # https://docs.pytest.org/en/latest/writing_plugins.html#making-your-plugin-installable-by-others
    entry_points={"pytest11": ["kubetest = kubetest.plugin"]},
)
